@@ -4,7 +4,7 @@ import asyncio
 import time
 from http import HTTPStatus
 from types import SimpleNamespace
-from typing import Optional, Union
+from typing import Any, Optional, Union
 
 from docudevs.api.cases.create_case import asyncio_detailed as create_case_async
 from docudevs.api.cases.get_case import asyncio_detailed as get_case_async
@@ -46,6 +46,7 @@ from docudevs.api.operations.submit_operation import (
 from docudevs.api.template import delete_template, fill, list_templates, metadata
 from docudevs.client import AuthenticatedClient
 from docudevs.models import UploadCommand, UploadDocumentBody, OcrCommand, OcrType, LlmType
+from docudevs.models.map_reduce_header_command import MapReduceHeaderCommand
 from docudevs.models.extraction_mode import check_extraction_mode
 from docudevs.models.generative_task_request import GenerativeTaskRequest
 from docudevs.models.template_fill_request import TemplateFillRequest
@@ -60,6 +61,17 @@ class DocuDevsClient:
             raise ValueError("token is required")
         self._client = AuthenticatedClient(base_url=api_url, token=token)
 
+    def _normalize_page_indices(self, value: Any) -> list[int]:
+        if isinstance(value, (str, bytes)):
+            raise ValueError("header.page_indices must be a sequence of integers")
+        try:
+            normalized = [int(v) for v in value]
+        except TypeError as exc:
+            raise ValueError("header.page_indices must be a sequence of integers") from exc
+        except ValueError as exc:
+            raise ValueError("header.page_indices must contain only integers") from exc
+        return normalized
+
     async def list_configurations(self):
         """List all named configurations."""
         return await list_configurations.asyncio_detailed(client=self._client)
@@ -239,6 +251,9 @@ class DocuDevsClient:
         pages_per_chunk: int = 1,
         overlap_pages: int = 0,
         dedup_key: Optional[str] = None,
+        header_options: Optional[dict[str, Any]] = None,
+        header_schema: Optional[str] = None,
+        header_prompt: Optional[str] = None,
     ) -> UploadCommand:
         """Build an UploadCommand with map-reduce (chunked) parameters.
 
@@ -257,6 +272,50 @@ class DocuDevsClient:
         if overlap_pages > 0 and not dedup_key:
             raise ValueError("dedup_key is required when overlap_pages > 0")
 
+        header_model: Union[MapReduceHeaderCommand, Unset]
+        header_model = UNSET
+        if header_options is not None or header_schema is not None or header_prompt is not None:
+            opts: dict[str, Any] = dict(header_options or {})
+            if "page_indices" not in opts and "pageIndices" in opts:
+                opts["page_indices"] = opts["pageIndices"]
+            if "page_indices" in opts and opts["page_indices"] is not None:
+                opts["page_indices"] = self._normalize_page_indices(opts["page_indices"])
+            enabled = bool(opts.get("enabled", True))
+            raw_page_limit = opts.get("page_limit")
+            if enabled:
+                effective_page_limit = raw_page_limit if raw_page_limit is not None else 1
+                if effective_page_limit < 1:
+                    raise ValueError("header.page_limit must be >= 1 when header is enabled")
+            else:
+                effective_page_limit = raw_page_limit
+                if effective_page_limit is not None and effective_page_limit < 1:
+                    raise ValueError("header.page_limit must be >= 1 when provided")
+
+            if header_schema is not None:
+                opts["schema"] = header_schema
+            if header_prompt is not None:
+                opts["prompt"] = header_prompt
+
+            def _value_or_unset(key: str) -> Union[Any, Unset]:
+                if key not in opts:
+                    return UNSET
+                value = opts[key]
+                if value is None:
+                    return UNSET
+                return value
+
+            header_model = MapReduceHeaderCommand(
+                enabled=enabled,
+                page_limit=effective_page_limit if effective_page_limit is not None else UNSET,
+                include_in_rows=_value_or_unset("include_in_rows"),
+                page_indices=_value_or_unset("page_indices"),
+                schema=_value_or_unset("schema"),
+                schema_file_name=_value_or_unset("schema_file_name"),
+                prompt=_value_or_unset("prompt"),
+                prompt_file_name=_value_or_unset("prompt_file_name"),
+                row_prompt_augmentation=_value_or_unset("row_prompt_augmentation"),
+            )
+
         if extraction_mode is not None:
             extraction_mode_value = check_extraction_mode(extraction_mode)
         else:
@@ -281,6 +340,8 @@ class DocuDevsClient:
             overlap_pages=overlap_pages,
             dedup_key=dedup_key if dedup_key else UNSET,
         )
+        if not isinstance(header_model, Unset):
+            mr_model.header = header_model
         cmd.map_reduce = mr_model
         # Add camelCase alias attribute for backward/compat tests expecting getattr(cmd, "mapReduce")
         mr_dict = mr_model.to_dict()
@@ -315,8 +376,53 @@ class DocuDevsClient:
         pages_per_chunk: int = 1,
         overlap_pages: int = 0,
         dedup_key: Optional[str] = None,
+        header_options: Optional[dict[str, Any]] = None,
+        header_schema: Optional[str] = None,
+        header_prompt: Optional[str] = None,
     ):
         """Process an already-uploaded document using map-reduce chunking parameters."""
+
+        def _fallback_header_payload() -> dict[str, Any] | None:
+            if (
+                header_options is None
+                and header_schema is None
+                and header_prompt is None
+            ):
+                return None
+            opts: dict[str, Any] = dict(header_options or {})
+            if "page_indices" not in opts and "pageIndices" in opts:
+                opts["page_indices"] = opts["pageIndices"]
+            if "page_indices" in opts and opts["page_indices"] is not None:
+                opts["page_indices"] = self._normalize_page_indices(opts["page_indices"])
+            enabled = bool(opts.get("enabled", True))
+            raw_page_limit = opts.get("page_limit")
+            effective_page_limit = (
+                raw_page_limit if raw_page_limit is not None else 1
+                if enabled
+                else raw_page_limit
+            )
+            header_payload: dict[str, Any] = {"enabled": enabled}
+            if effective_page_limit is not None:
+                header_payload["pageLimit"] = effective_page_limit
+            if "include_in_rows" in opts and opts.get("include_in_rows") is not None:
+                header_payload["includeInRows"] = opts["include_in_rows"]
+            if header_schema is not None:
+                header_payload["schema"] = header_schema
+            if "schema_file_name" in opts and opts.get("schema_file_name"):
+                header_payload["schemaFileName"] = opts["schema_file_name"]
+            if header_prompt is not None:
+                header_payload["prompt"] = header_prompt
+            if "prompt_file_name" in opts and opts.get("prompt_file_name"):
+                header_payload["promptFileName"] = opts["prompt_file_name"]
+            if (
+                "row_prompt_augmentation" in opts
+                and opts.get("row_prompt_augmentation")
+            ):
+                header_payload["rowPromptAugmentation"] = opts["row_prompt_augmentation"]
+            if "page_indices" in opts and opts.get("page_indices") is not None:
+                header_payload["pageIndices"] = opts["page_indices"]
+            return header_payload
+
         cmd = self.build_upload_command_map_reduce(
             prompt=prompt,
             schema=schema,
@@ -329,6 +435,9 @@ class DocuDevsClient:
             pages_per_chunk=pages_per_chunk,
             overlap_pages=overlap_pages,
             dedup_key=dedup_key,
+            header_options=header_options,
+            header_schema=header_schema,
+            header_prompt=header_prompt,
         )
 
         # Ensure JSON body includes mapReduce block even if model ignored attribute
@@ -352,19 +461,27 @@ class DocuDevsClient:
                         except Exception:
                             pass
                 else:
-                    setattr(cmd, "mapReduce", {
+                    base_map_reduce = {
                         "enabled": True,
                         "pagesPerChunk": pages_per_chunk,
                         "overlapPages": overlap_pages,
                         **({"dedupKey": dedup_key} if dedup_key else {}),
-                    })
+                    }
+                    header_dict = _fallback_header_payload()
+                    if header_dict:
+                        base_map_reduce["header"] = header_dict
+                    setattr(cmd, "mapReduce", base_map_reduce)
             except Exception:
-                setattr(cmd, "mapReduce", {
+                header_dict = _fallback_header_payload()
+                base_map_reduce = {
                     "enabled": True,
                     "pagesPerChunk": pages_per_chunk,
                     "overlapPages": overlap_pages,
                     **({"dedupKey": dedup_key} if dedup_key else {}),
-                })
+                }
+                if header_dict:
+                    base_map_reduce["header"] = header_dict
+                setattr(cmd, "mapReduce", base_map_reduce)
         return await process_document_async(client=self._client, guid=guid, body=cmd)
 
     async def submit_and_process_document_map_reduce(
@@ -382,6 +499,9 @@ class DocuDevsClient:
         pages_per_chunk: int = 1,
         overlap_pages: int = 0,
         dedup_key: Optional[str] = None,
+        header_options: Optional[dict[str, Any]] = None,
+        header_schema: Optional[str] = None,
+        header_prompt: Optional[str] = None,
     ) -> str:
         """Upload a document then process it using map-reduce chunking.
 
@@ -416,6 +536,9 @@ class DocuDevsClient:
             pages_per_chunk=pages_per_chunk,
             overlap_pages=overlap_pages,
             dedup_key=dedup_key,
+            header_options=header_options,
+            header_schema=header_schema,
+            header_prompt=header_prompt,
         )
         if process_resp.status_code != HTTPStatus.OK:
             content_str = process_resp.content.decode("utf-8", errors="replace")
@@ -1124,17 +1247,32 @@ class DocuDevsClient:
         document,
         document_mime_type: str,
         ocr: str = "DEFAULT",
-        ocr_format: str = "markdown",
+        ocr_format: Optional[str] = None,
         describe_figures: Optional[bool] = None,
     ) -> str:
+        """Convenience helper for OCR-only runs.
+
+        Args:
+            document: File-like object or bytes to upload.
+            document_mime_type: MIME type for the uploaded document.
+            ocr: OCR mode to invoke (e.g. "DEFAULT", "EXCEL").
+            ocr_format: Desired OCR output format. ``None`` defaults to ``"markdown"`` for text OCR
+                and ``"jsonl"`` for Excel mode.
+            describe_figures: Whether to request figure descriptions when supported.
+        """
         if not document_mime_type:
             raise ValueError("document_mime_type is required")
         if document is None:
             raise ValueError("document is required")
         # Allow LOW OCR to ignore describe_figures per server semantics; for others, plain+describe_figures is invalid
         ocr_upper = (ocr or "DEFAULT").upper()
-        if describe_figures is True and ocr_format == "plain" and ocr_upper != "LOW":
+        effective_format = ocr_format or ("jsonl" if ocr_upper == "EXCEL" else "markdown")
+        if describe_figures is True and effective_format == "plain" and ocr_upper != "LOW":
             raise ValueError("describe_figures=True is not supported with ocr_format='plain'")
+        if effective_format == "jsonl" and ocr_upper != "EXCEL":
+            raise ValueError("ocr_format='jsonl' is only supported with ocr='EXCEL'")
+        if effective_format not in {"markdown", "plain", "jsonl"}:
+            raise ValueError("Invalid ocr_format; expected 'markdown', 'plain', or 'jsonl'")
         file_obj = File(payload=document, file_name="omitted", mime_type=document_mime_type)
         upload_body = UploadDocumentBody(document=file_obj)
         upload_response = await self.upload_document(body=upload_body)
@@ -1149,14 +1287,14 @@ class DocuDevsClient:
         ocr_enum = check_ocr_type(ocr) if ocr is not None else UNSET
         ocr_body = OcrCommand(
             ocr=ocr_enum,
-            ocr_format=ocr_format,
+            ocr_format=effective_format,
             mime_type=document_mime_type,
             # For LOW OCR, ignore describe_figures by not sending it; server also ignores it defensively
             describe_figures=(
                 UNSET if ocr_upper == "LOW" else (describe_figures if describe_figures is not None else UNSET)
             ),
         )
-        ocr_resp = await self.ocr_document(guid=guid, body=ocr_body, ocr_format=ocr_format)
+        ocr_resp = await self.ocr_document(guid=guid, body=ocr_body, ocr_format=effective_format)
         if ocr_resp.status_code != HTTPStatus.OK:
             content_str = ocr_resp.content.decode("utf-8", errors="replace")
             raise Exception(f"Error processing document with OCR: {content_str}")
